@@ -1,11 +1,19 @@
 package Chart::Clicker::Axis;
+$Chart::Clicker::Axis::VERSION = '2.90';
 use Moose;
+use Moose::Util;
 
 extends 'Chart::Clicker::Container';
 with 'Chart::Clicker::Positioned';
 
+# ABSTRACT: An X or Y Axis
+
+use Class::Load;
+
 use Chart::Clicker::Data::Range;
 
+use English qw(-no_match_vars);
+
 use Graphics::Color::RGB;
 
 use Graphics::Primitive::Font;
@@ -14,15 +22,25 @@ use Layout::Manager::Absolute;
 
 use Math::Trig ':pi';
 
+
 has 'tick_label_angle' => (
-    is => 'rw',
+    is  => 'rw',
     isa => 'Num'
 );
+
+
+has 'tick_division_type' => ( is => 'rw', isa => 'Str', default => 'Exact' );
+
+# The above tick division type is loaded on the first call to divvy()
+has '_tick_division_type_loaded' => ( is => 'ro', isa => 'Bool', lazy_build => 1 );
+
+
 has 'baseline' => (
     is  => 'rw',
     isa => 'Num',
 );
-# Remove for border...
+
+
 has 'brush' => (
     is => 'rw',
     isa => 'Graphics::Primitive::Brush',
@@ -35,6 +53,8 @@ has 'brush' => (
         )
     }
 );
+
+
 has '+color' => (
     default => sub {
         Graphics::Color::RGB->new({
@@ -42,10 +62,20 @@ has '+color' => (
         })
     }
 );
-has 'format' => ( is => 'rw', isa => 'Str|CodeRef' );
+
+
+has 'format' => ( is => 'rw', isa => 'Str|CodeRef', default => '%s' );
+
+
 has 'fudge_amount' => ( is => 'rw', isa => 'Num', default => 0 );
+
+
 has 'hidden' => ( is => 'rw', isa => 'Bool', default => 0 );
+
+
 has 'label' => ( is => 'rw', isa => 'Str' );
+
+
 has 'label_color' => (
     is => 'rw',
     isa => 'Graphics::Color',
@@ -55,35 +85,55 @@ has 'label_color' => (
         })
     }
 );
+
+
 has 'label_font' => (
     is => 'rw',
     isa => 'Graphics::Primitive::Font',
     default => sub { Graphics::Primitive::Font->new }
 );
+
+
 has '+layout_manager' => ( default => sub { Layout::Manager::Absolute->new });
+
+
 has '+orientation' => (
     required => 1
 );
+
+
 has '+position' => (
     required => 1
 );
+
+
 has 'range' => (
     is => 'rw',
     isa => 'Chart::Clicker::Data::Range',
     default => sub { Chart::Clicker::Data::Range->new }
 );
+
+
 has 'show_ticks' => ( is => 'rw', isa => 'Bool', default => 1 );
+
+
 has 'staggered' => ( is => 'rw', isa => 'Bool', default => 0 );
+
+
 has 'skip_range' => (
     is => 'rw',
     isa => 'Chart::Clicker::Data::Range',
     predicate => 'has_skip_range'
 );
+
+
 has 'tick_font' => (
     is => 'rw',
     isa => 'Graphics::Primitive::Font',
     default => sub { Graphics::Primitive::Font->new }
 );
+
+
 has 'tick_label_color' => (
     is => 'rw',
     isa => 'Graphics::Color',
@@ -93,10 +143,14 @@ has 'tick_label_color' => (
         })
     }
 );
+
+
 has 'tick_labels' => (
     is => 'rw',
     isa => 'ArrayRef',
 );
+
+
 has 'tick_values' => (
     traits => [ 'Array' ],
     is => 'rw',
@@ -108,7 +162,9 @@ has 'tick_values' => (
         'tick_value_count' => 'count'
     }
 );
-has 'ticks' => ( is => 'rw', isa => 'Int', default => 5 );
+
+
+has 'ticks' => ( is => 'rw', isa => 'Int', default => 6 );
 
 sub BUILD {
     my ($self) = @_;
@@ -116,6 +172,32 @@ sub BUILD {
     $self->padding(3);
 }
 
+sub _build__tick_division_type_loaded {
+    my $self = shift;
+
+    # User modules are prefixed with a +.
+    my $divisionTypeModule;
+    my $extensionOf = 'Chart::Clicker::Axis::DivisionType';
+    if ( $self->tick_division_type =~ m/^\+(.*)$/xmisg ) {
+        $divisionTypeModule = $1;
+    }
+    else {
+        $divisionTypeModule = sprintf( '%s::%s', $extensionOf, $self->tick_division_type );
+    }
+
+    # Try to load the DivisionType module. An error is thrown when the class is
+    # not available or cannot be loaded
+    Class::Load::load_class($divisionTypeModule);
+
+    # Apply the newly loaded role to this class.
+    Moose::Util::apply_all_roles( $self => $divisionTypeModule );
+    if ( not $self->does($extensionOf) ) {
+        die("Module $divisionTypeModule does not extend $extensionOf");
+    }
+
+    return 1;
+}
+
 override('prepare', sub {
     my ($self, $driver) = @_;
 
@@ -156,10 +238,10 @@ override('prepare', sub {
     }
 
     if($self->show_ticks && !scalar(@{ $self->tick_values })) {
-        $self->tick_values($self->range->divvy($self->ticks + 1));
+        $self->tick_values($self->divvy);
     }
 
-    # Return now without setting a min height or width and allow 
+    # Return now without setting a min height or width and allow
     # Layout::Manager to to set it for us, this is how we 'hide'
     return if $self->hidden;
 
@@ -179,7 +261,12 @@ override('prepare', sub {
 
         my $label = $val;
         if(defined($self->tick_labels)) {
-            $label = $self->tick_labels->[$i];
+            if (defined $self->tick_labels->[$i]) {
+                $label = $self->tick_labels->[$i];
+            }
+            else {
+                $label = "";
+            }
         } else {
             $label = $self->format_value($val);
         }
@@ -215,15 +302,18 @@ override('prepare', sub {
     my $label_height = 0;
 
     if ($self->label) {
-
         my $label = Graphics::Primitive::TextBox->new(
             # angle => $angle,
             color => $self->label_color,
             name => 'label',
             font => $self->label_font,
             text => $self->label,
-            width => $self->height
+            width => $self->height,
+            horizontal_align => 'center'
         );
+        $label->name('label');
+        $label->border->color(Graphics::Color::RGB->new(r => 0, g => 0, b => 0));
+
 
         if($self->is_vertical) {
             if ($self->is_left) {
@@ -245,11 +335,15 @@ override('prepare', sub {
     }
 
     if($self->is_vertical) {
-        $self->minimum_width($self->outside_width + $big + $label_width);
-        $self->minimum_height($self->outside_height + $self->outside_height + $big);
+        my $new_min_width = $self->outside_width + $big + $label_width;
+        $self->minimum_width($new_min_width) if $new_min_width > $self->minimum_width;
+        my $new_min_height = $self->outside_height + $big;
+        $self->minimum_height($new_min_height) if $new_min_height > $self->minimum_height;
     } else {
-        $self->minimum_height($self->outside_height + $big + $label_height);
-        $self->minimum_width($self->outside_width + $big + $self->outside_width);
+        my $new_min_height = $self->outside_height + $big + $label_height;
+        my $new_min_width = $self->outside_width + $big;
+        $self->minimum_height($new_min_height)  if $new_min_height > $self->minimum_height;
+        $self->minimum_width($new_min_width) if $new_min_width > $self->minimum_width;
         if($self->staggered) {
             $self->minimum_height($self->minimum_height * 2);
         }
@@ -258,9 +352,11 @@ override('prepare', sub {
     return 1;
 });
 
+
 sub mark {
     my ($self, $span, $value) = @_;
 
+    return undef if not defined $value;
     if($self->has_skip_range) {
         # We must completely ignore values that fall inside the skip range,
         # so we return an undef.
@@ -292,9 +388,11 @@ sub mark {
 override('finalize', sub {
     my ($self) = @_;
 
-    super;
-
-    return if $self->hidden;
+    if($self->hidden) {
+        # Call the callback, just in case it matters.
+        super;
+        return;
+    }
 
     my $x = 0;
     my $y = 0;
@@ -333,33 +431,28 @@ override('finalize', sub {
             my $iy = $height - $mark;
             my $label = $self->get_component($comp_count);
 
+            # Adjust text on the Y axis to fit when it is
+            # too close to the edges
+            my $standardYOrigin = $iy - ($label->height / 2);
+            #my $lowerYOrigin = $iy - $label->height;
+            my $lowerYOrigin = $ioy + $iheight - $label->height;
+            my $upperYOrigin = $ioy;
+            if($standardYOrigin >= $lowerYOrigin) {
+                # The first label (being at the bottom) needs to be
+                # skooched up a bit to fit.
+                $label->origin->y($lowerYOrigin);
+            } elsif($standardYOrigin <= $upperYOrigin) {
+                # The last label (being at the top) can be positioned
+                # exactly at the mark.
+                $label->origin->y($upperYOrigin);
+            } else {
+                $label->origin->y($standardYOrigin);
+            }
+
             if($self->is_left) {
                 $label->origin->x($iox + $iwidth - $label->width);
-                if($val == $lower) {
-                    # The first label (being at the bottom) needs to be
-                    # skooched up a bit to fit.
-                    $label->origin->y($iy - $label->height);
-                } elsif($val == $upper) {
-                    # The last label (being at the top) can be positioned
-                    # exactly at the mark.
-                    $label->origin->y($iy);
-                } else {
-                    $label->origin->y($iy - ($label->height / 2));
-                }
             } else {
-
                 $label->origin->x($iox);
-                if($val == $lower) {
-                    # The first label (being at the bottom) needs to be
-                    # skooched up a bit to fit.
-                    $label->origin->y($iy - $label->height);
-                } elsif($val == $upper) {
-                    # The last label (being at the top) can be positioned
-                    # exactly at the mark.
-                    $label->origin->y($iy);
-                } else {
-                    $label->origin->y($iy - ($label->height / 2));
-                }
             }
             # Keep track of how many components we've actually grabbed, since
             # we could be skipping any that are in a skip range.
@@ -398,24 +491,22 @@ override('finalize', sub {
                 }
             }
 
-            if($self->is_top) {
+            # Adjust the X-origin value to ensure it is within the graph
+            # and does not get snipped when too close to the edge.
+            my $standardXOrigin = $ix - ($label->width / 1.8);
+            my $lowerXOrigin = $iox;
+            my $upperXOrigin = $iox + $iwidth - $label->width;
+            if($standardXOrigin <= $lowerXOrigin) {
+                $label->origin->x($lowerXOrigin);
+            } elsif($standardXOrigin >= $upperXOrigin) {
+                $label->origin->x($upperXOrigin);
+            } else {
+                $label->origin->x($standardXOrigin);
+            }
 
-                if($val == $lower) {
-                    $label->origin->x($ix);
-                } elsif($val == $upper) {
-                    $label->origin->x($ix - $label->width);
-                } else {
-                    $label->origin->x($ix - ($label->width / 1.8));
-                }
+            if($self->is_top) {
                 $label->origin->y($ioy + $iheight - $label->height - $bump);
             } else {
-                if($val == $lower) {
-                    $label->origin->x($ix);
-                } elsif($val == $upper) {
-                    $label->origin->x($ix - $label->width);
-                } else {
-                    $label->origin->x($ix - ($label->width / 1.8));
-                }
                 $label->origin->y($ioy + $bump);
             }
             # Keep track of how many components we've actually grabbed, since
@@ -440,8 +531,11 @@ override('finalize', sub {
             }
         }
     }
+
+    super;
 });
 
+
 sub format_value {
     my $self = shift;
     my $value = shift;
@@ -457,20 +551,33 @@ sub format_value {
     }
 }
 
+
+sub divvy {
+    my $self = shift;
+
+    # Loads the divvy module once and only once
+    # which implements _real_divvy()
+    $self->_tick_division_type_loaded;
+    return $self->_real_divvy();
+}
+
 __PACKAGE__->meta->make_immutable;
 
 no Moose;
 
 1;
+
 __END__
 
+=pod
+
 =head1 NAME
 
 Chart::Clicker::Axis - An X or Y Axis
 
-=head1 DESCRIPTION
+=head1 VERSION
 
-Chart::Clicker::Axis represents the plot of the chart.
+version 2.90
 
 =head1 SYNOPSIS
 
@@ -479,15 +586,47 @@ Chart::Clicker::Axis represents the plot of the chart.
   use Graphics::Primitive::Brush;
 
   my $axis = Chart::Clicker::Axis->new({
-    font  => Graphics::Primitive::Font->new,
+    label_font  => Graphics::Primitive::Font->new,
     orientation => 'vertical',
     position => 'left',
     brush => Graphics::Primitive::Brush->new,
     visible => 1,
   });
 
+=head1 DESCRIPTION
+
+Chart::Clicker::Axis represents the plot of the chart.
+
 =head1 ATTRIBUTES
 
+=head2 height
+
+The height of the axis.
+
+=head2 width
+
+This axis' width.
+
+=head2 tick_label_angle
+
+The angle (in radians) to rotate the tick's labels.
+
+=head2 tick_division_type
+
+Selects the algorithm for dividing the graph axis into labelled ticks.
+
+The currently included algorithms are:
+L<Chart::Clicker::Data::DivisionType::Exact/Exact>,
+L<Chart::Clicker::Data::DivisionType::LinearRounded/LinearRounded>.
+
+You may write your own by providing a Moose Role which includes Role
+L<Chart::Clicker::Data::DivisionType> and prefixing the module name
+with + when setting tick_division_type.
+
+ Chart::Clicker::Axis->new(tick_division_type => '+MyApp::TickDivision');
+
+This value should only be set once per axis.
+
 =head2 baseline
 
 The 'baseline' value of this axis.  This is used by some renderers to change
@@ -496,11 +635,12 @@ below the base to be 'negative'.
 
 =head2 brush
 
-The brush for this axis.
+The brush for this axis.  Expects a L<Graphics::Primitve::Brush>.
 
 =head2 color
 
-The color of the axis' border.
+The color of the axis' border.  Expects a L<Graphics::Color::RGB> object.
+Defaults to black.
 
 =head2 format
 
@@ -526,10 +666,6 @@ below the dataset.
 As an example, a fugdge_amount of .10 on an axis with a span of 10 to 50
 would add 5 to the top and bottom of the axis.
 
-=head2 height
-
-The height of the axis.
-
 =head2 hidden
 
 This axis' hidden flag.  If this is true then the Axis will not be drawn.
@@ -540,11 +676,15 @@ The label of the axis.
 
 =head2 label_color
 
-The color of the Axis' labels.
+The color of the Axis' labels. Expects a L<Graphics::Color::RGB> object.
 
 =head2 label_font
 
-The font used for the axis' label.
+The font used for the axis' label. Expects a L<Graphics::Primitive::Font> object.
+
+=head2 layout_manager
+
+Set/Get the Layout Manager.  Defaults to L<Layout::Manager::Absolute>.
 
 =head2 orientation
 
@@ -556,13 +696,24 @@ The position of the axis on the chart.
 
 =head2 range
 
-The Range for this axis.
+The Range for this axis. Expects a L<Chart::Clicker::Data::Range> object.
+You may use this to explicitly set an upper and lower bound for the chart:
+
+  $axis->range->max(1000);
+  $axis->range->min(1);
 
 =head2 show_ticks
 
 If this is value is false then 'ticks' and their labels will not drawn for
 this axis.
 
+=head2 staggered
+
+If true, causes horizontally labeled axes to 'stagger' the labels so that half
+are at the top of the box and the other half are at the bottom.  This makes
+long, overlapping labels less likely to overlap.  It only does something
+useful with B<horizontal> labels.
+
 =head2 skip_range
 
 Allows you to specify a range of values that will be skipped completely on
@@ -573,36 +724,17 @@ attribute will make for a much more useful chart, albeit somewhat visually
 skewed.
 
   $axis->skip_range(Chart::Clicker::Data::Range->new(lower => 10, upper => 100));
-  
+
 Note that B<any> data points, including ticks, that fall inside the range
 specified will be completely ignored.
 
-=head2 stagger
-
-If true, causes horizontally labeled axes to 'stagger' the labels so that half
-are at the top of the box and the other half are at the bottom.  This makes
-long, overlapping labels less likely to overlap.  It only does something
-useful with B<horizontal> labels.
-
 =head2 tick_font
 
-The font used for the axis' ticks.
-
-=head2 tick_label_angle
-
-The angle (in radians) to rotate the tick's labels.
+The font used for the axis' ticks. Expects a L<Graphics::Primitive::Font> object.
 
 =head2 tick_label_color
 
-The color of the tick labels.
-
-=head2 tick_values
-
-The arrayref of values show as ticks on this Axis.
-
-=head2 tick_value_count
-
-Get a count of tick values.
+The color of the tick labels. Expects a L<Graphics::Color::RGB> object.
 
 =head2 tick_labels
 
@@ -610,23 +742,18 @@ The arrayref of labels to show for ticks on this Axis.  This arrayref is
 consulted for every tick, in order.  So placing a string at the zeroeth index
 will result in it being displayed on the zeroeth tick, etc, etc.
 
+=head2 tick_values
+
+The arrayref of values show as ticks on this Axis.
+
 =head2 ticks
 
 The number of 'ticks' to show.  Setting this will divide the range on this
 axis by the specified value to establish tick values.  This will have no
 effect if you specify tick_values.
 
-=head2 width
-
-This axis' width.
-
 =head1 METHODS
 
-=head2 new
-
-Creates a new Chart::Clicker::Axis.  If no arguments are given then sane
-defaults are chosen.
-
 =head2 add_to_tick_values
 
 Add a value to the list of tick values.
@@ -635,6 +762,10 @@ Add a value to the list of tick values.
 
 Clear all tick values.
 
+=head2 tick_value_count
+
+Get a count of tick values.
+
 =head2 mark
 
 Given a span and a value, returns it's pixel position on this Axis.
@@ -643,29 +774,19 @@ Given a span and a value, returns it's pixel position on this Axis.
 
 Given a value, returns it formatted using this Axis' formatter.
 
-=head2 prepare
+=head2 divvy
 
-Prepare this Axis by determining the size required.  If the orientation is
-CC_HORIZONTAL this method sets the height.  Otherwise sets the width.
-
-=head2 draw
-
-Draw this axis.
-
-=head2 BUILD
-
-Documening for POD::Coverage tests, Moose stuff.
+Retrieves the divisions or ticks for the axis.
 
 =head1 AUTHOR
 
 Cory G Watson <gphat@cpan.org>
 
-=head1 SEE ALSO
+=head1 COPYRIGHT AND LICENSE
 
-perl(1)
+This software is copyright (c) 2016 by Cory G Watson.
 
-=head1 LICENSE
-
-You can redistribute and/or modify this code under the same terms as Perl
-itself.
+This is free software; you can redistribute it and/or modify it under
+the same terms as the Perl 5 programming language system itself.
 
+=cut
